from injector import singleton, provides, Module, inject
from lazy import lazy

try:
    import configparser
except ImportError:
    import ConfigParser as configparser


@singleton
class Config(object):
    @inject(config_parser=configparser.ConfigParser)
    def __init__(self, config_parser):
        self.config_parser = config_parser

    @lazy
    def client_location_name(self):
        return self.config_parser.get('client', 'location_name')

    @lazy
    def store_username(self):
        return self.config_parser.get('store', 'username')

    @lazy
    def store_password(self):
        return self.config_parser.get('store', 'password')

    @lazy
    def store_name(self):
        return self.config_parser.get('store', 'name')

    @lazy
    def store_host(self):
        return self.config_parser.get('store', 'host')

    @lazy
    def store_port(self):
        return int(self.config_parser.get('store', 'port'))

    @lazy
    def log_path(self):
        return self.config_parser.get('log', 'path')


class ConfigModule(Module):
    @singleton
    @provides(configparser.ConfigParser)
    def provide_config_parser(self):
        config_parser = configparser.ConfigParser()
        config_parser.read('/etc/klimalogger/klimalogger.conf')
        return config_parser



from injector import singleton, inject, Injector

from .data_log import DataLog
from .sensor_factory import SensorFactory
from .store_client import StoreClient
from .data_builder import DataBuilder
from .config import ConfigModule

INJECTOR = Injector(
    [config.ConfigModule()])


@singleton
class Client(object):
    @inject(data_builder=DataBuilder, sensor_factory=SensorFactory, store_client=StoreClient, data_log=DataLog)
    def __init__(self, data_builder, sensor_factory, store_client, data_log):
        self.data_builder = data_builder
        self.sensor_factory = sensor_factory
        self.store_client = store_client
        self.data_log = data_log

    def measure_and_store(self):
        self.sensor_factory.measure(self.data_builder)

        data = self.data_builder.data
        timestamp = self.data_builder.timestamp
        try:
            self.store_client.store(data)
            print("stored data")
        except:
            print("error during data transmission: create local log entry")
            self.data_log.store(data, timestamp)

        self.data_log.transmit_stored_data()


def client():
    return INJECTOR.get(Client)



from injector import singleton, inject, Injector

import importlib

try:
    import configparser
except ImportError:
    import ConfigParser as configparser

@singleton
class SensorFactory(object):
    @inject(configuration=configparser.ConfigParser, current_injector=Injector)
    def __init__(self, configuration, current_injector):
        self.sensors = [sensor.strip() for sensor in configuration.get('client', 'sensors').split(',')]
        self.current_injector = current_injector

    def measure(self, data_builder):
        for sensor in self.sensors:
            module = importlib.import_module('klimalogger.sensor.' + sensor + '_sensor')
            print("module:", module)
            sensor = self.current_injector.get(module.Sensor)
            sensor.measure(data_builder)



from injector import singleton, inject

from influxdb import InfluxDBClient

from . import config


@singleton
class StoreClient(object):
    @inject(configuration=config.Config)
    def __init__(self, configuration):
        try:
            self.client = InfluxDBClient(
                host=configuration.store_host,
                port=configuration.store_port,
                database=configuration.store_name,
                username=configuration.store_username,
                password=configuration.store_password,
                timeout=5)
        except RuntimeError as e:
            print("could not create client", e)
            self.client = None

    def store(self, data):
        if self.client:
            print("write data")
            self.client.write_points(data)
        else:
            print("client not available")
            raise RuntimeError("bla")



def client():
    from . import INJECTOR

    return INJECTOR.get(StoreClient)
